package com.flozano.mim.air.dao.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.inject.Inject;
import javax.sql.DataSource;

import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import com.flozano.mim.air.Accrual;
import com.flozano.mim.air.AirportCode;
import com.flozano.mim.air.BookingClass;
import com.flozano.mim.air.dao.AccrualDAO;

@Repository
public class JDBCAccrualDAOImpl implements AccrualDAO {

	private final JdbcTemplate jdbcTemplate;

	@Inject
	public JDBCAccrualDAOImpl(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	@Override
	public Accrual get(AirportCode origin, AirportCode destination,
			BookingClass bookingClass) {
		List<Accrual> result = jdbcTemplate
				.query("SELECT * FROM accrual WHERE origin=? AND destination=? AND booking_class=?",
						ROW_MAPPER, origin.toString(), destination.toString(),
						bookingClass.toString());
		switch (result.size()) {
		case 0:
			return null;
		case 1:
			return result.get(0);
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	private static final RowMapper<Accrual> ROW_MAPPER = new RowMapper<Accrual>() {

		@Override
		public Accrual mapRow(ResultSet rs, int rowNum) throws SQLException {
			Accrual a = new Accrual();
			a.setBookingClass(new BookingClass(rs.getString("booking_class")));
			a.setDestination(new AirportCode("destination"));
			a.setId(rs.getLong("id"));
			a.setMiles(rs.getInt("miles"));
			a.setOrigin(new AirportCode(rs.getString("origin")));
			return a;
		}
	};

}
