package com.flozano.mim.air.dao.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import com.flozano.mim.air.Airline;
import com.flozano.mim.air.AirlineCode;
import com.flozano.mim.air.AirlineNotFoundException;
import com.flozano.mim.air.dao.AirlineDAO;

@Repository
public class JDBCAirlineDAOImpl implements AirlineDAO {
	private final JdbcTemplate jdbcTemplate;

	@Autowired
	public JDBCAirlineDAOImpl(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	@Override
	public Airline get(AirlineCode code) throws AirlineNotFoundException {
		List<Airline> a = jdbcTemplate.query(
				"SELECT id, name FROM airline WHERE id = ?", ROW_MAPPER,
				code.toString());
		switch (a.size()) {
		case 0:
			throw new AirlineNotFoundException(code);
		case 1:
			return a.get(0);
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	@Override
	public List<Airline> findAll() {
		return jdbcTemplate.query("SELECT * FROM airline ORDER BY name",
				ROW_MAPPER);
	}

	private static final RowMapper<Airline> ROW_MAPPER = new RowMapper<Airline>() {

		@Override
		public Airline mapRow(ResultSet rs, int i) throws SQLException {
			Airline a = new Airline();
			a.setCode(new AirlineCode(rs.getString(1)));
			a.setName(rs.getString(2));
			return a;
		}

	};

}
