package com.flozano.mim.air.dao.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.inject.Inject;
import javax.sql.DataSource;

import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import com.flozano.mim.air.Airport;
import com.flozano.mim.air.AirportCode;
import com.flozano.mim.air.AirportNotFoundException;
import com.flozano.mim.air.dao.AirportDAO;
import com.flozano.mim.country.CountryID;

@Repository
public class JDBCAirportDAOImpl implements AirportDAO {

	private final JdbcTemplate jdbcTemplate;

	@Inject
	public JDBCAirportDAOImpl(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	@Override
	public Airport get(AirportCode code) throws AirportNotFoundException {
		List<Airport> a = jdbcTemplate.query(
				"SELECT id, country, name FROM airport WHERE id = ?",
				ROW_MAPPER, code.toString());
		switch (a.size()) {
		case 0:
			throw new AirportNotFoundException(code);
		case 1:
			return a.get(0);
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	@Override
	public List<Airport> findByCountry(CountryID countryID) {
		return jdbcTemplate
				.query("SELECT id, country, name FROM airport WHERE country = ? ORDER BY id",
						ROW_MAPPER, countryID.toString());
	}

	@Override
	public List<Airport> findAll() {
		return jdbcTemplate.query(
				"SELECT id, country, name FROM airport ORDER BY country, id",
				ROW_MAPPER);
	}

	private static final RowMapper<Airport> ROW_MAPPER = new RowMapper<Airport>() {

		@Override
		public Airport mapRow(ResultSet rs, int i) throws SQLException {
			Airport a = new Airport();
			a.setId(new AirportCode(rs.getString(1)));
			a.setCountry(new CountryID(rs.getString(2)) );
			a.setName(rs.getString(3));
			return a;
		}

	};

}
