package com.flozano.mim.air.dao.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.inject.Inject;
import javax.sql.DataSource;

import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import com.flozano.mim.air.AirportCode;
import com.flozano.mim.air.Lounge;
import com.flozano.mim.air.LoungeNotFoundException;
import com.flozano.mim.air.dao.LoungeDAO;

@Repository
public class JDBCLoungeDAOImpl implements LoungeDAO {

	private static final String SQL_SELECT_ALL = "SELECT * FROM lounge ORDER BY airport, name";
	private static final String SQL_SELECT_BY_AIRPORT = "SELECT * FROM lounge WHERE airport = ? ORDER BY name";

	private final JdbcTemplate jdbcTemplate;

	@Inject
	public JDBCLoungeDAOImpl(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	@Override
	public void insert(AirportCode airport, String name, String description) {
		switch (jdbcTemplate
				.update("INSERT INTO lounge (airport, name, description) VALUES (?, ?, ?)",
						airport, name, description)) {
		case 1:
			return;
		case 0:
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	@Override
	public void update(long loungeId, AirportCode airportCode, String name,
			String description) throws LoungeNotFoundException {
		switch (jdbcTemplate
				.update("UPDATE lounge SET airport=?, name=?, description=? WHERE id = ?",
						airportCode.toString(), name, description, loungeId)) {
		case 0:
			throw new LoungeNotFoundException(loungeId);
		case 1:
			return;
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	@Override
	public void delete(long loungeId) throws LoungeNotFoundException {
		switch (jdbcTemplate
				.update("DELETE FROM lounge WHERE id = ?", loungeId)) {
		case 0:
			throw new LoungeNotFoundException(loungeId);
		case 1:
			return;
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	@Override
	public List<Lounge> find(AirportCode airportCode) {
		if (airportCode != null) {
			return jdbcTemplate.query(SQL_SELECT_BY_AIRPORT, ROW_MAPPER,
					airportCode.toString());
		}
		return jdbcTemplate.query(SQL_SELECT_ALL, ROW_MAPPER);
	}

	private static final RowMapper<Lounge> ROW_MAPPER = new RowMapper<Lounge>() {

		@Override
		public Lounge mapRow(ResultSet rs, int rowNum) throws SQLException {
			Lounge l = new Lounge();
			l.setId(rs.getLong("id"));
			l.setAirportCode(new AirportCode(rs.getString("airport")));
			l.setDescription(rs.getString("description"));
			l.setName(rs.getString("name"));
			return l;
		}
	};

	@Override
	public Lounge get(long loungeId) {
		List<Lounge> result = jdbcTemplate.query(
				"SELECT * FROM lounge WHERE id = ?", ROW_MAPPER, loungeId);
		switch (result.size()) {
		case 0:
			throw new LoungeNotFoundException(loungeId);
		case 1:
			return result.get(0);
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}
}
