package com.flozano.mim.air.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.flozano.mim.air.AirService;
import com.flozano.mim.air.Airline;
import com.flozano.mim.air.AirlineCode;
import com.flozano.mim.air.Airport;
import com.flozano.mim.air.AirportCode;
import com.flozano.mim.air.dao.AirlineDAO;
import com.flozano.mim.air.dao.AirportDAO;
import com.flozano.mim.country.CountryID;
import com.flozano.mim.util.Check;

@Service("airService")
@Transactional
public class AirServiceImpl implements AirService {
	private final AirlineDAO airlineDAO;

	private final AirportDAO airportDAO;

	@Autowired
	public AirServiceImpl(AirlineDAO airlineDAO, AirportDAO airportDAO) {
		Check.notNull(airlineDAO, airportDAO);
		this.airlineDAO = airlineDAO;
		this.airportDAO = airportDAO;
	}

	@Override
	public Airline getAirline(AirlineCode code) {
		if (code == null) {
			throw new IllegalArgumentException();
		}
		return airlineDAO.get(code);
	}

	@Override
	public Airport getAirport(AirportCode code) {
		if (code == null) {
			throw new IllegalArgumentException();
		}
		return airportDAO.get(code);
	}

	@Override
	public List<Airport> findAirports(CountryID countryID) {
		return airportDAO.findByCountry(countryID);
	}

	@Override
	public List<Airport> findAllAirports() {
		return airportDAO.findAll();
	}

	@Override
	public List<Airline> findAllAirlines() {
		return airlineDAO.findAll();
	}

}
