package com.flozano.mim.air.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.flozano.mim.air.AirportCode;
import com.flozano.mim.air.Lounge;
import com.flozano.mim.air.LoungeNotFoundException;
import com.flozano.mim.air.LoungeService;
import com.flozano.mim.air.dao.LoungeDAO;
import com.flozano.mim.util.Check;

@Transactional
@Service("loungeService")
public class LoungeServiceImpl implements LoungeService {

	private final LoungeDAO loungeDAO;

	@Autowired
	public LoungeServiceImpl(LoungeDAO loungeDAO) {
		Check.notNull(loungeDAO);
		this.loungeDAO = loungeDAO;
	}

	@Override
	public List<Lounge> find(AirportCode airportCode) {
		return loungeDAO.find(airportCode);
	}

	@Override
	public void create(String name, String description, AirportCode airportCode) {
		loungeDAO.insert(airportCode, name, description);
	}

	@Override
	public void update(Lounge lounge) throws LoungeNotFoundException {
		loungeDAO.update(lounge.getId(), lounge.getAirportCode(),
				lounge.getName(), lounge.getDescription());
	}

	@Override
	public void delete(long loungeId) throws LoungeNotFoundException {
		loungeDAO.delete(loungeId);
	}

	@Override
	public Lounge get(long loungeId) {
		return loungeDAO.get(loungeId);
	}
}
