package com.flozano.mim.member;

import org.displaytag.pagination.PaginatedList;

import com.flozano.mim.MemberNumber;
import com.flozano.mim.util.SortDirection;

public interface MemberService {

	/**
	 * Registers a new user in the system
	 * 
	 * @param fullName
	 * @param password
	 * @param emailAddress
	 * @param phoneNumber
	 * @param postalAddress
	 * @return
	 */
	MemberNumber register(FullName fullName, Password password,
			EmailAddress emailAddress, PhoneNumber phoneNumber,
			PostalAddress postalAddress);

	/**
	 * Updates the user password
	 * 
	 * @param memberNumber
	 * @param newPassword
	 */
	void changePassword(MemberNumber memberNumber, Password oldPassword,
			Password newPassword);

	/**
	 * Obtains a member
	 * 
	 * @param memberNumber
	 * @return
	 * @throws MemberNotFoundException
	 */
	Member getMember(MemberNumber memberNumber) throws MemberNotFoundException;

	/**
	 * Obtains a member
	 * 
	 * @param memberId
	 * @return
	 */
	Member getMember(long memberId) throws MemberNotFoundException;

	/**
	 * Update the member (update'able information only)
	 * 
	 * @param memberNumber
	 * @param emailAddress
	 * @param phoneNumber
	 * @param address
	 */
	void update(MemberNumber memberNumber, EmailAddress emailAddress,
			PhoneNumber phoneNumber, PostalAddress address);

	/**
	 * Update the member (all information)
	 * 
	 * @param memberId
	 * @param fullName
	 * @param emailAddress
	 * @param phoneNumber
	 * @param postalAddress
	 */
	void update(long memberId, FullName fullName, EmailAddress emailAddress,
			PhoneNumber phoneNumber, PostalAddress postalAddress);

	/**
	 * 
	 * @param search
	 * @param sortProperty
	 * @param direction
	 * @param page
	 * @return
	 */
	PaginatedList find(String search, String sortProperty,
			SortDirection direction, int page);

}
