package com.flozano.mim.member.dao.jdbc;

import java.sql.Types;
import java.util.Date;

import javax.inject.Inject;
import javax.sql.DataSource;

import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.object.SqlUpdate;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.stereotype.Repository;

import com.flozano.mim.MemberNumber;
import com.flozano.mim.member.dao.MemberAccountStatementDAO;

@Repository
public class JDBCMemberAccountStatementDAOImpl implements
		MemberAccountStatementDAO {

	private static final String SQL_INSERT = "INSERT INTO member_account_statement (member, applies_at, miles) VALUES (?, ?, ?)";
	private static final int[] SQL_INSERT_TYPES = new int[] { Types.BIGINT,
			Types.TIMESTAMP, Types.INTEGER };
	private static final String[] SQL_INSERT_GENERATED_KEYS = new String[] { "id" };

	private final SqlUpdate insertOperation;

	private final JdbcTemplate jdbcTemplate;

	@Inject
	public JDBCMemberAccountStatementDAOImpl(DataSource dataSource) {
		jdbcTemplate = new JdbcTemplate(dataSource);
		insertOperation = new SqlUpdate(dataSource, SQL_INSERT,
				SQL_INSERT_TYPES, 1);
		insertOperation.setGeneratedKeysColumnNames(SQL_INSERT_GENERATED_KEYS);
	}

	@Override
	public long insert(long memberId, Date date, int miles) {
		GeneratedKeyHolder h = new GeneratedKeyHolder();
		int n = insertOperation.update(new Object[] { memberId, date, miles },
				h);
		if (n != 1)
			throw new IncorrectResultSizeDataAccessException(1);
		return (Long) h.getKey();
	}

	@Override
	public int totalMiles(long memberId) {
		return jdbcTemplate
				.queryForInt(
						"SELECT SUM(miles) FROM member_account_statement WHERE member=?",
						memberId);
	}

	@Override
	public int sumMilesAccruedBetween(MemberNumber memberNumber, Date from,
			Date until) {
		return jdbcTemplate
				.queryForInt(
						"SELECT SUM(miles) FROM member_account_statement s, member m WHERE s.miles > 0 AND m.member_number=? AND m.id=s.member AND applies_at BETWEEN ? AND ?",
						memberNumber.toString(), from, until);
	}

	@Override
	public int totalMiles(MemberNumber memberNumber) {
		return jdbcTemplate
				.queryForInt(
						"SELECT SUM(miles) FROM member_account_statement s, member m WHERE m.member_number=? AND m.id=s.member",
						memberNumber.toString());
	}

	@Override
	public int sumMilesAccruedBetween(long memberId, Date from, Date until) {
		return jdbcTemplate
				.queryForInt(
						"SELECT SUM(miles) FROM member_account_statement WHERE miles > 0 AND member=? AND applies_at BETWEEN ? AND ?",
						memberId, from, until);
	}

	@Override
	public void updateMilesBySegment(long segmentId, int miles) {
		int n = jdbcTemplate
				.update("UPDATE member_account_statement m, segment s SET m.miles=? WHERE s.member_account_statement=m.id AND s.id=?",
						miles, segmentId);
		if (n != 1) {
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

}
