package com.flozano.mim.member.dao.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.List;

import javax.inject.Inject;
import javax.sql.DataSource;

import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import com.flozano.mim.member.Note;
import com.flozano.mim.member.dao.NoteDAO;
import com.flozano.mim.util.Check;

@Repository
public class JDBCNoteDAOImpl implements NoteDAO {

	private final JdbcTemplate jdbcTemplate;
	private static final RowMapper<Note> ROW_MAPPER = new RowMapper<Note>() {

		@Override
		public Note mapRow(ResultSet rs, int rowNum) throws SQLException {
			Note n = new Note();
			n.setId(rs.getLong("id"));
			n.setContent(rs.getString("content"));
			n.setCreatedAt(rs.getDate("created_at"));
			return n;
		}
	};

	@Inject
	public JDBCNoteDAOImpl(DataSource dataSource) {
		Check.notNull(dataSource);
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	@Override
	public List<Note> findNotes(long memberId) {
		return jdbcTemplate.query(
				"SELECT * FROM note WHERE member = ? ORDER BY created_at DESC",
				ROW_MAPPER, memberId);
	}

	@Override
	public void addNote(long memberId, String content) {
		if (jdbcTemplate
				.update("INSERT INTO note (member, created_at, content) VALUES (?, ?, ?)",
						memberId, new Date(), content) != 1) {
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}
}
