package com.flozano.mim.member.impl;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import com.flozano.mim.MemberNumber;
import com.flozano.mim.air.AirlineCode;
import com.flozano.mim.air.AirportCode;
import com.flozano.mim.air.BookingClass;
import com.flozano.mim.air.Segment;

/**
 * 2012-05-23 VLC MAD IB 0320 I 9239238923232945
 * 
 * 2012-05-22 MAD BIO YW 8293 J 9239990530923945
 * 
 * @author flozano
 * 
 */
public class LineToSegmentTransformer {

	private static final String FORMAT = "yyyy-MM-dd";

	public Segment transform(String line) {
		SimpleDateFormat sdf = new SimpleDateFormat(FORMAT);
		Segment s = new Segment();
		String[] parts = line.trim().split("\t");
		if (parts.length != 7) {
			throw new IllegalArgumentException("Line is not valid: " + line);
		}
		try {
			s.setDeparture(sdf.parse(parts[0]));
		} catch (ParseException e) {
			throw new IllegalArgumentException(e);
		}
		s.setOrigin(new AirportCode(parts[1]));
		s.setDestination(new AirportCode(parts[2]));
		s.setAirline(new AirlineCode(parts[3]));
		s.setFlightNumber(Integer.parseInt(parts[4]));
		s.setBookingClass(new BookingClass(parts[5]));
		s.setLoadedMemberNumber(new MemberNumber(parts[6]));
		return s;
	}
}
