package com.flozano.mim.web.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;

import com.flozano.mim.air.AirService;
import com.flozano.mim.air.AirportCode;
import com.flozano.mim.air.LoungeService;
import com.flozano.mim.util.Check;

@Controller
public class LoungeBrowserController {

	private final LoungeService loungeService;
	private final AirService airService;

	@Autowired
	public LoungeBrowserController(LoungeService loungeService, AirService airService) {
		Check.notNull(loungeService, airService);
		this.loungeService = loungeService;
		this.airService = airService;
	}

	@RequestMapping("/member/lounges")
	public String get(Model model, @ModelAttribute GetForm form,
			BindingResult result) {
		String airport = form != null ? form.getAirport() : null;
		model.addAttribute("form", form == null ? new GetForm() : form);
		model.addAttribute("airports", airService.findAllAirports());
		model.addAttribute("lounges", loungeService
				.find((airport == null || airport.equals("-")) ? null
						: new AirportCode(airport)));
		return "lounge/list";
	}

	public static class GetForm {

		private String airport;

		public String getAirport() {
			return airport;
		}

		public void setAirport(String airport) {
			this.airport = airport;
		}
	}
}
