package com.flozano.mim.web.controllers;

import java.security.Principal;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.hibernate.validator.constraints.Email;
import org.hibernate.validator.constraints.Length;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.flozano.mim.AppException;
import com.flozano.mim.MemberNumber;
import com.flozano.mim.member.EmailAddress;
import com.flozano.mim.member.Member;
import com.flozano.mim.member.MemberService;
import com.flozano.mim.member.PhoneNumber;
import com.flozano.mim.member.PostalAddress;

@Controller
@RequestMapping("/member/profile")
public class MemberProfileController {

	private final MemberService memberService;

	@Autowired
	public MemberProfileController(MemberService memberService) {
		this.memberService = memberService;
	}

	@RequestMapping(method = RequestMethod.GET)
	public void profile(
			Model model,
			@RequestParam(value = "updated", required = false, defaultValue = "false") Boolean updated,
			Principal principal) {
		Member m = memberService
				.getMember(new MemberNumber(principal.getName()));
		model.addAttribute("updated", updated);
		model.addAttribute("member", m);
		model.addAttribute("form", new ProfileUpdateForm(m));
	}

	@RequestMapping(method = RequestMethod.POST)
	public String profile(Model model,
			@ModelAttribute("form") @Valid ProfileUpdateForm form,
			BindingResult result, Principal principal,
			RedirectAttributes redirectAttributes) {
		if (!result.hasErrors()) {
			try {
				memberService.update(new MemberNumber(principal.getName()),
						new EmailAddress(form.getEmailAddress()),
						new PhoneNumber(form.getPhoneNumber()),
						form.getPostalAddress());
				redirectAttributes.addAttribute("updated", Boolean.TRUE);
				return "redirect:/member/profile";

			} catch (AppException e) {
				result.addError(new ObjectError("form", e.getLocalizedMessage()));
			}
		}
		Member m = memberService
				.getMember(new MemberNumber(principal.getName()));

		model.addAttribute("member", m);
		return "member/profile";
	}

	public static class ProfileUpdateForm {
		@Email
		@NotNull
		@Length(min = 6, max = 250)
		private String emailAddress;
		@Size(min = 6, max = 15)
		private String phoneNumber;
		@Valid
		private PostalAddress postalAddress = new PostalAddress();

		public ProfileUpdateForm() {
		}

		public ProfileUpdateForm(Member m) {
			this.emailAddress = m.getEmailAddress().toString();
			this.phoneNumber = m.getPhoneNumber().toString();
			this.postalAddress = m.getPostalAddress(); // TODO copy deeper -
														// cleaner...
		}

		public String getEmailAddress() {
			return emailAddress;
		}

		public void setEmailAddress(String emailAddress) {
			this.emailAddress = emailAddress;
		}

		public String getPhoneNumber() {
			return phoneNumber;
		}

		public void setPhoneNumber(String phoneNumber) {
			this.phoneNumber = phoneNumber;
		}

		public PostalAddress getPostalAddress() {
			return postalAddress;
		}

		public void setPostalAddress(PostalAddress postalAddress) {
			this.postalAddress = postalAddress;
		}

	}
}
