package com.flozano.mim.web.controllers;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.hibernate.validator.constraints.Email;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.ScriptAssert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.flozano.mim.AppException;
import com.flozano.mim.MemberNumber;
import com.flozano.mim.member.EmailAddress;
import com.flozano.mim.member.FullName;
import com.flozano.mim.member.MemberService;
import com.flozano.mim.member.Password;
import com.flozano.mim.member.PhoneNumber;
import com.flozano.mim.member.PostalAddress;

@Controller
@RequestMapping("/register")
public class RegisterController {

	private final MemberService memberService;

	@Autowired
	public RegisterController(MemberService memberService) {
		this.memberService = memberService;
	}

	@RequestMapping(method = RequestMethod.GET)
	public String register(Model model) {
		model.addAttribute("form", new RegistrationForm());
		System.out.println("Hola hola cocacola");
		return "register/form";
	}

	@RequestMapping(method = RequestMethod.POST)
	public String register(
			@Valid @ModelAttribute("form") RegistrationForm form,
			BindingResult result, RedirectAttributes redirectAttributes) {
		if (!result.hasErrors()) {
			try {
				MemberNumber memberNumber = memberService.register(
						form.getFullName(), new Password(form.getPassword()),
						new EmailAddress(form.getEmailAddress()),
						new PhoneNumber(form.getPhoneNumber()),
						form.getPostalAddress());
				redirectAttributes.addAttribute("memberNumber",
						memberNumber.toString());
				return "redirect:/register/result";

			} catch (AppException e) {
				result.addError(new ObjectError("form", e.getLocalizedMessage()));
			}
		}
		return "register/form";
	}

	@RequestMapping(value = "result", method = RequestMethod.GET)
	public String registrationResult(Model model,
			@ModelAttribute("memberNumber") String memberNumber,
			RedirectAttributes redirectAttributes) {
		
		// redirectAttributes.addAttribute("memberNumber",
		// model.asMap().get("memberNumber"));
		return "register/result";
	}

	@ScriptAssert(lang = "javascript", script = "(_this.password!=null) && (_this.password.equals(_this.repeatPassword) )", message = "Passwords do not match")
	public static class RegistrationForm {
		@Valid
		private FullName fullName = new FullName();

		@Length(min = 6, max = 50)
		@NotNull
		private String password;

		@Length(min = 6, max = 50)
		private String repeatPassword;

		@Email
		@NotNull
		@Length(min = 6, max = 250)
		private String emailAddress;

		@Size(min = 6, max = 15)
		private String phoneNumber;

		@Valid
		private PostalAddress postalAddress = new PostalAddress();

		public FullName getFullName() {
			return fullName;
		}

		public void setFullName(FullName fullName) {
			this.fullName = fullName;
		}

		public String getEmailAddress() {
			return emailAddress;
		}

		public void setEmailAddress(String emailAddress) {
			this.emailAddress = emailAddress;
		}

		public String getPhoneNumber() {
			return phoneNumber;
		}

		public void setPhoneNumber(String phoneNumber) {
			this.phoneNumber = phoneNumber;
		}

		public PostalAddress getPostalAddress() {
			return postalAddress;
		}

		public void setPostalAddress(PostalAddress postalAddress) {
			this.postalAddress = postalAddress;
		}

		public String getPassword() {
			return password;
		}

		public void setPassword(String password) {
			this.password = password;
		}

		public String getRepeatPassword() {
			return repeatPassword;
		}

		public void setRepeatPassword(String repeatPassword) {
			this.repeatPassword = repeatPassword;
		}

	}
}
