package com.flozano.mim.web.controllers.admin;

import java.util.Date;

import javax.inject.Inject;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.flozano.mim.AppException;
import com.flozano.mim.air.AirService;
import com.flozano.mim.air.AirportCode;
import com.flozano.mim.air.Lounge;
import com.flozano.mim.air.LoungeService;
import com.flozano.mim.util.Check;
import com.flozano.mim.web.controllers.LoungeBrowserController.GetForm;
import com.flozano.mim.web.controllers.admin.MemberController.UpdateMemberForm;

@Controller
@RequestMapping("/admin/lounges")
public class LoungeController {

	private final LoungeService loungeService;
	private final AirService airService;

	@Inject
	public LoungeController(LoungeService loungeService, AirService airService) {
		Check.notNull(loungeService, airService);
		this.loungeService = loungeService;
		this.airService = airService;
	}

	@RequestMapping(method = RequestMethod.GET)
	public String list(Model model, @ModelAttribute GetForm form,
			BindingResult result) {
		String airport = form != null ? form.getAirport() : null;
		model.addAttribute("form", form == null ? new GetForm() : form);
		model.addAttribute("airports", airService.findAllAirports());
		model.addAttribute("lounges", loungeService
				.find((airport == null || airport.equals("-")) ? null
						: new AirportCode(airport)));
		return "admin/lounges/list";
	}

	@RequestMapping(method = RequestMethod.GET, value = "/{loungeId}")
	public String getLounge(
			Model model,
			@PathVariable("loungeId") long loungeId,
			@RequestParam(required = false, defaultValue = "false") Boolean loungeUpdated) {
		Lounge lounge = loungeService.get(loungeId);
		model.addAttribute("loungeForm", new LoungeForm(lounge));
		model.addAttribute("loungeUpdated", loungeUpdated);
		model.addAttribute("airports", airService.findAllAirports());
		return "admin/lounges/form";
	}

	@RequestMapping(method = RequestMethod.POST, value = "/{loungeId}")
	public String updateLounge(
			@ModelAttribute("loungeForm") LoungeForm loungeForm,
			BindingResult bindingResult,
			RedirectAttributes redirectAttributes,
			Model model,
			@PathVariable long loungeId,
			@RequestParam(required = false, defaultValue = "2000-01-01") @DateTimeFormat(pattern = "yyyy-MM-dd") Date from,
			@RequestParam(required = false, defaultValue = "2050-12-31") @DateTimeFormat(pattern = "yyyy-MM-dd") Date until) {
		try {
			Lounge lounge = loungeService.get(loungeId);
			lounge.setName(loungeForm.getName());
			lounge.setAirportCode(new AirportCode(loungeForm.getAirportCode()));
			lounge.setDescription(loungeForm.getDescription());
			loungeService.update(lounge);
			redirectAttributes.addAttribute("loungeUpdated", Boolean.TRUE);
			return "redirect:/admin/lounges";
		} catch (AppException e) {
			bindingResult.addError(new ObjectError("loungeForm", e
					.getLocalizedMessage()));
		}
		return "admin/lounges/edit";
	}

	public static class LoungeForm {
		@Size(min = 3, max = 3)
		private String airportCode;
		@Size(min = 3, max = 50)
		@NotNull
		private String name;
		@Size(min = 5, max = 500)
		@NotNull
		private String description;

		public LoungeForm() {
		}

		public LoungeForm(Lounge lounge) {
			this.airportCode = lounge.getAirportCode().toString();
			this.name = lounge.getName();
			this.description = lounge.getDescription();
		}

		public String getAirportCode() {
			return airportCode;
		}

		public void setAirportCode(String airportCode) {
			this.airportCode = airportCode;
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public String getDescription() {
			return description;
		}

		public void setDescription(String description) {
			this.description = description;
		}

	}
}
